#include <iostream>
#include <string>
using namespace std;

#include "simpio.h"
#include "random.h"
#include "gwindow.h"
#include "grid.h"
static const int kNumDigits = 9;
static const int kBoardDimension = kNumDigits;
static const int kBlockWidth = kNumDigits / 3;
static const double kSquareSize = 48;
static const double kWindowDimension = (kBoardDimension + 1) * kSquareSize;
static const int kEmpty = 0;

static const double kInset = 2;
static const double kWindowInset = 10;
static const string kPossiblilityColor = "#aaaaaa";
static const string kProvisionalColor = "#0000ff";
static const string kPermanantColor = "#55aa55";
static const string kNoPossibilityColor = "#ffffff";
static const double kDelay = 1;
static const string kBoard[] = {
    "000030200",
    "020040350",
    "007100008",
    "800001062",
    "060000070",
    "270600003",
    "700008900",
    "056090040",
    "008010000"
};

static void drawSquare(GWindow& window, int row, int col, double inset = 0) {
    int ulx = (window.getWidth() - kBoardDimension * kSquareSize)/2 + col * kSquareSize;
    int uly = (window.getHeight() - kBoardDimension * kSquareSize)/2 + row * kSquareSize;
    window.setColor("White");
    window.fillRect(ulx + inset, uly + inset,
                    kSquareSize - 2 * inset, kSquareSize - 2 * inset);
    window.setColor("Gray");
    window.drawRect(ulx + inset, uly + inset,
                    kSquareSize - 2 * inset, kSquareSize - 2 * inset);
}

static void placeNumber(GWindow& window, int number, int row, int col, string color) {
    drawSquare(window, row, col, kInset);
    if (number == kEmpty) return;
    double cx = (window.getWidth() - kBoardDimension * kSquareSize)/2 + col * kSquareSize + kSquareSize/2;
    double cy = (window.getHeight() - kBoardDimension * kSquareSize)/2 + row * kSquareSize + kSquareSize/2;
    window.setColor(color);
    string label = integerToString(number);
    window.drawString(label, cx - window.getStringWidth(label)/2, cy + window.getFontAscent()/2);
}
static double getSeparatorX(GWindow& window, int col) {
    return (window.getWidth() - kBoardDimension * kSquareSize)/2 + col * kSquareSize;
}
static double getSeparatorY(GWindow& window, int row) {
    return (window.getHeight() - kBoardDimension * kSquareSize)/2 + row * kSquareSize;
}
static void drawSeparators(GWindow& window) {
    window.setColor("Black");
    window.drawLine(getSeparatorX(window, 3), kWindowInset,
                    getSeparatorX(window, 3), window.getWidth() - kWindowInset);
    window.drawLine(getSeparatorX(window, 6), kWindowInset,
                    getSeparatorX(window, 6), window.getWidth() - kWindowInset);
    window.drawLine(kWindowInset, getSeparatorY(window, 3),
                    window.getHeight() - kWindowInset, getSeparatorY(window, 3));
    window.drawLine(kWindowInset, getSeparatorY(window, 6),
                    window.getHeight() - kWindowInset, getSeparatorY(window, 6));
}

static void configureBoard(GWindow& window, Grid<int>& board) {
	for (int row = 0; row < kBoardDimension; row++) {
		for (int col = 0; col < kBoardDimension; col++) {
			board[row][col] = kBoard[row][col] - '0';
            placeNumber(window, board[row][col], row, col, "Black");
		}
	}
    
    drawSeparators(window);
}
static void updateBoardLocation(GWindow& window, Grid<int>& board,
                                int row, int col, int digit) {
    board[row][col] = digit;
    placeNumber(window, digit, row, col, "Blue");
    pause(kDelay);
}

static bool findEmptyLocation(Grid<int>& board, int& row, int& col) {
    for (row = 0; row < kBoardDimension; row++) {
        for (col = 0; col < kBoardDimension; col++) {
            if (board[row][col] == kEmpty) {
                return true;
            }
        }
    }
    
    return false;
}

static bool isFreeOfConflict(const Grid<int>& board, int row, int col, int digit) {
    for (int c = 0; c < kBoardDimension; c++) {
        if (board.get(row, c) == digit) return false;
    }
    
    for (int r = 0; r < kBoardDimension; r++) {
        if (board.get(r, col) == digit) return false;
    }
    
    int rowblock = row / kBlockWidth;
    int colblock = col / kBlockWidth;
    for (int drow = 0; drow < kBlockWidth; drow++) {
        for (int dcol = 0; dcol < kBlockWidth; dcol++) {
            if (board.get(kBlockWidth * rowblock + drow, kBlockWidth * colblock + dcol) == digit) {
                return false;
            }
        }
    }
    
    return true;
}

static int countNumOptions(Grid<int>& board, int row, int col) {
    int numOptions = 0;
    for (int digit = 1; digit <= kNumDigits; digit++) {
        if (isFreeOfConflict(board, row, col, digit))
            numOptions++;
    }
    
    return numOptions;
}

static bool findBestEmptyLocation(Grid<int>& board, int& row, int& col) {
    int smallestNumOptions = kNumDigits + 1;
    for (int r = 0; r < kBoardDimension; r++) {
        for (int c = 0; c < kBoardDimension; c++) {
            if (board[r][c] == kEmpty) {
                int numOptions = countNumOptions(board, r, c);
                if (numOptions < smallestNumOptions) {
                    row = r;
                    col = c;
                    smallestNumOptions = numOptions;
                }
            }
        }
    }
    
    return smallestNumOptions <= kNumDigits;
}

static bool solve(GWindow& window, Grid<int>& board) {
    int row, col;
    if (!findEmptyLocation(board, row, col)) return true;
    
    for (int digit = 1; digit <= 9; digit++) {
        if (isFreeOfConflict(board, row, col, digit)) {
            updateBoardLocation(window, board, row, col, digit);
            if (solve(window, board)) return true;
            updateBoardLocation(window, board, row, col, kEmpty);
        }
    }
    
    return false;
}

int main() {
    GWindow window(kWindowDimension, kWindowDimension);
	Grid<int> board(kBoardDimension, kBoardDimension);
    window.setWindowTitle("Solve SuDoKu");
    window.setFont("Times-Bold-" + integerToString(0.7 * kSquareSize));
    configureBoard(window, board);
    waitForClick();
    solve(window, board);
    return 0;
}
